@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005-2006 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,6 +42,7 @@
 #include <string.h>
 #include <hildon-widgets/hildon-caption.h>
 #include <hildon-widgets/hildon-defines.h>
+#include <hildon-widgets/hildon-banner.h>
 #include <libintl.h>
 #include <gdk/gdkx.h>
 #include <stdlib.h>
@@ -150,6 +150,7 @@ struct _HildonFileChooserDialogPrivate {
     gint max_full_path_length;
     gint max_filename_length;
     gboolean popup_protect;
+    gchar *infobanner_message;
 
     /* Popup menu contents */
     GtkWidget *sort_type, *sort_name, *sort_date, *sort_size;
@@ -163,6 +164,7 @@ struct _HildonFileChooserDialogPrivate {
     gboolean edited;
     gboolean should_show_folder_button;
     gboolean should_show_location;
+    gboolean show_upnp;
 };
 
 static void hildon_file_chooser_dialog_iface_init(GtkFileChooserIface *
@@ -201,6 +203,17 @@ unescape_character (const char *scanner)
   return (first_digit << 4) | second_digit;
 }
 
+static void chooser_entry_invalid_input_cb (GtkEntry *entry,
+                                            GtkInvalidInputType inv_type,
+                                            gpointer user_data)
+{
+  if (inv_type == GTK_INVALID_INPUT_MAX_CHARS_REACHED) {
+    /* show the infoprint "ckdg_ib_maximum_characters_reached" */
+    hildon_banner_show_information (GTK_WIDGET (entry), NULL,
+				    HCS("ckdg_ib_maximum_characters_reached"));
+  }
+}
+
 static gchar *
 g_unescape_uri_string (const char *escaped,
                        int         len,
@@ -294,75 +307,27 @@ get_path_length_from_uri(const char *uri)
 static void
 hildon_file_chooser_dialog_set_limit(HildonFileChooserDialog *self)
 {
-  gchar *uri;
-  const char *str;
-  gint max_length = 0, len;
-  gboolean limited;
+  /* The full pathname is limited to MAX_FULL_PATH_LENGTH characters,
+     thus we could try to be smart here and limit the length of the
+     input field to MAX_FULL_PATH_LENGTH minus the length of the
+     current folder pathname.  That, however, leads to quite wierd
+     behavior when we get close to the limit, or even beyond it,
+     especially when the user changes the current folder.  Instead, we
+     allow arbitrary entry lengths and validate the full pathname
+     length when the user hits "Ok".
+  */
 
-  /* We cannot get the current folder uri without filetree, 
-     which is not available during initial setup */
-  if (self->priv->max_full_path_length >= 0 && self->priv->filetree)
-  {
-    uri = gtk_file_chooser_get_current_folder_uri(GTK_FILE_CHOOSER(self));
-    max_length = self->priv->max_full_path_length - get_path_length_from_uri(uri);
-    g_free(uri);
-
-    /* Possible extension length must be subtracted from maximum 
-       possible length */
-    if ((str = self->priv->ext_name) != NULL)
-    {
-      len = g_utf8_strlen(str, -1);
-      max_length -= len;
-      ULOG_DEBUG("Extension = [%s], length = %d", str, len);
-    }
-
-    /* We have to alloc space for trailing slash */
-    max_length--;
-
-    limited = TRUE;
-  }
-  else
-    limited = FALSE;
-
-  if (self->priv->max_filename_length >= 0 && 
-     (!limited || self->priv->max_filename_length < max_length))
-  {
-    max_length = self->priv->max_filename_length;
-    limited = TRUE;
-  }
-
-  if (limited)
-  {
-    /* Maximum length 0 means "no limit" for entry and negatives are
-       not allowed.  Setting this to one at least. */
-    if (max_length < 1) max_length = 1;
-
-    ULOG_DEBUG("Setting maximum length to %d", max_length);
-    gtk_entry_set_max_length(GTK_ENTRY(self->priv->entry_name), max_length);
-  }
-  else
-  {
-    ULOG_DEBUG("Unsetting maximum length");
-    gtk_entry_set_max_length(GTK_ENTRY(self->priv->entry_name), 0);
-  }
-}
-
-static void 
-hildon_file_chooser_dialog_check_limit(GtkEditable *editable,
-  gchar *new_text, gint new_text_length,
-  gint *position, gpointer data)
-{
-  GtkEntry *entry = GTK_ENTRY(editable);
-
-  if (entry->text_max_length > 0 && 
-      entry->text_length + new_text_length > entry->text_max_length)
-      gtk_infoprint(GTK_WINDOW(data), 
-          HCS("ckdg_ib_maximum_characters_reached"));  
+  ULOG_DEBUG ("Setting maximum length to %d", max_length);
+  gtk_entry_set_max_length (GTK_ENTRY (self->priv->entry_name),
+			    self->priv->max_filename_length);
 }
 
 static void insensitive_button(GtkWidget *widget, gpointer data)
 { 
-    gtk_infoprint(GTK_WINDOW(data), _("sfil_ib_select_file"));
+  HildonFileChooserDialogPrivate *priv =
+	HILDON_FILE_CHOOSER_DIALOG(data)->priv;
+
+  hildon_banner_show_information(widget, NULL, priv->infobanner_message);
 }
 
 static void file_activated_handler(GtkWidget * widget, gpointer user_data)
@@ -421,9 +386,11 @@ hildon_file_chooser_dialog_do_autonaming(HildonFileChooserDialogPrivate *
 
         if (name)
         {
+	  gboolean edited = !selection && 
+		  	     (pos != g_utf8_strlen (gtk_entry_get_text(GTK_ENTRY(priv->entry_name)), -1));
           gtk_entry_set_text(GTK_ENTRY(priv->entry_name), name);
           g_free(name);
-          if (selection)
+          if (!edited)
             gtk_editable_select_region(GTK_EDITABLE(priv->entry_name), 0, -1);
           else
             /* if the user has already started to edit the name,
@@ -775,6 +742,7 @@ static void build_ui(HildonFileChooserDialog * self)
                              _("ckdg_bd_select_object_ok_open"));
         gtk_button_set_label(GTK_BUTTON(priv->cancel_button),
                              _("ckdg_bd_select_object_cancel"));
+	priv->infobanner_message = _("sfil_ib_select_file");
         break;
     case GTK_FILE_CHOOSER_ACTION_SAVE:
         if (hildon_file_chooser_dialog_save_multiple_set(priv))
@@ -812,6 +780,7 @@ static void build_ui(HildonFileChooserDialog * self)
                              _("ckdg_bd_save_object_dialog_cancel"));
         gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(self),
                                           _("ckdg_va_save_object_name_stub_default"));
+	priv->infobanner_message = HCS("ckdg_ib_enter_name");
         break;
     case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
         gtk_widget_hide(priv->caption_control_name);
@@ -828,6 +797,7 @@ static void build_ui(HildonFileChooserDialog * self)
                              _("ckdg_bd_change_folder_new_folder"));
         gtk_button_set_label(GTK_BUTTON(priv->cancel_button),
                              _("ckdg_bd_change_folder_cancel"));
+	priv->infobanner_message = _("sfil_ib_select_file");
         break;
     case GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER:
         hildon_caption_set_label(HILDON_CAPTION(priv->caption_control_name),
@@ -838,13 +808,15 @@ static void build_ui(HildonFileChooserDialog * self)
         gtk_widget_hide(GTK_WIDGET(priv->filetree));
 	_hildon_file_selection_realize_help(priv->filetree);        
         gtk_widget_hide(priv->hbox_items);
-        gtk_window_set_title(GTK_WINDOW(self), _("ckdg_ti_new_folder"));
+        gtk_window_set_title(GTK_WINDOW(self), dgettext("hildon-libs",
+                             "ckdg_ti_new_folder"));
         gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(self),
                                           _("ckdg_va_new_folder_name_stub")); 
         gtk_button_set_label(GTK_BUTTON(priv->action_button),
                              _("ckdg_bd_new_folder_dialog_ok"));
         gtk_button_set_label(GTK_BUTTON(priv->cancel_button),
                              _("ckdg_bd_new_folder_dialog_cancel"));
+	priv->infobanner_message = HCS("ckdg_ib_enter_name");
         break;
     default:
         g_assert_not_reached();
@@ -861,22 +833,18 @@ static GString *check_illegal_characters(const gchar * name)
 {
     GString *illegals = NULL;
 
-    /* Handle reserved names as special cases */
-    if (strcmp(name, ".") == 0 || strcmp(name, "..") == 0)
-      return g_string_new(name);
-
     while (*name) {
         gunichar unichar;
 
         unichar = g_utf8_get_char(name);
 
-        if (unichar == '\\' || unichar == '/' || unichar == ':' || unichar == '*' || unichar == '?' ||
-            unichar == '\"' || unichar == '<' || unichar == '>' || unichar == '|' ) {
+	if (g_utf8_strchr ("\\/:*?\"<>|", -1, unichar))
+	  {
             if (!illegals)
                 illegals = g_string_new(NULL);
-            if (g_utf8_strchr(illegals->str, -1, unichar) == NULL)
-                g_string_append_unichar(illegals, unichar);
-        }
+            if (g_utf8_strchr (illegals->str, -1, unichar) == NULL)
+	      g_string_append_unichar (illegals, unichar);
+	  }
 
         name = g_utf8_next_char(name);
     }
@@ -998,7 +966,7 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
       else
         message = _("sfil_ib_create_folder_not_allowed");
 
-      ULOG_ERR_F(error->message);
+      ULOG_ERR_F("%s", error->message);
       gtk_infoprint(GTK_WINDOW(dialog), message);
       g_error_free(error);
 
@@ -1014,7 +982,6 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
       sync_current_folders(HILDON_FILE_CHOOSER_DIALOG(dialog), self);
   }
 
-  gtk_banner_close(GTK_WINDOW(dialog));
   gtk_widget_destroy(dialog);
   gtk_window_present(GTK_WINDOW(self));
   self->priv->popup_protect = FALSE;
@@ -1056,10 +1023,16 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
                 hildon_file_chooser_dialog_do_autonaming(priv);
                 hildon_file_chooser_dialog_select_text(priv);
                 gtk_infoprint(window, HCS("ckdg_ib_enter_name"));
-            } else {
+            } else if (entry_text[0] == '.') {
+	        /* We don't allow files with a dot as the first character.
+		 */
+                hildon_file_chooser_dialog_select_text(priv);
+	        g_signal_stop_emission_by_name(widget, "response");
+		gtk_infoprint(window, _("sfil_ib_invalid_name_dot"));
+	    } else {
                 GString *illegals = check_illegal_characters(entry_text);
 
-                if (illegals) {
+		if (illegals) {
                     gchar *msg;
 
                     hildon_file_chooser_dialog_select_text(priv);
@@ -1074,9 +1047,7 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
                 }
                 else if (self->priv->max_full_path_length >= 0)
                      {  /* Let's check that filename is not too long. 
-                           This normally should not happen, because we 
-                           have limit in entry. However, very long paths 
-                           could cause max input length to be less than 1. */
+			 */
                     gchar *uri;
                     gint path_length;
 
@@ -1335,7 +1306,7 @@ static void hildon_file_chooser_dialog_destroy(GtkObject * obj)
 {
     HildonFileChooserDialog *dialog;
  
-    ULOG_DEBUG(__FUNCTION__);
+    ULOG_DEBUG_F("entered");
 
     dialog = HILDON_FILE_CHOOSER_DIALOG(obj);
 
@@ -1386,7 +1357,7 @@ hildon_file_chooser_dialog_class_init(HildonFileChooserDialogClass * klass)
     gobject_class->constructor = hildon_file_chooser_dialog_constructor;
     gobject_class->finalize = hildon_file_chooser_dialog_finalize;
     GTK_WIDGET_CLASS(klass)->show = hildon_file_chooser_dialog_show;
-    GTK_WIDGET_CLASS(klass)->show_all = hildon_file_chooser_dialog_show;
+    GTK_WIDGET_CLASS(klass)->show_all = gtk_widget_show;
     GTK_OBJECT_CLASS(klass)->destroy = hildon_file_chooser_dialog_destroy;
 
     pspec = g_param_spec_string("empty-text", "Empty text",
@@ -1453,7 +1424,7 @@ hildon_file_chooser_dialog_class_init(HildonFileChooserDialogClass * klass)
                              "Use -1 for no limit or 0 to look the value "
                              "from MAX_FILENAME_LENGTH environment variable", 
                              -1, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
+    
     hildon_gtk_file_chooser_install_properties(gobject_class);
 }
 
@@ -1518,6 +1489,9 @@ hildon_file_chooser_entry_changed( GtkWidget * widget,
 
     priv = HILDON_FILE_CHOOSER_DIALOG( data )->priv;
     priv->edited = TRUE;
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(data),
+		    		      GTK_RESPONSE_OK,
+    				      strlen(gtk_entry_get_text(GTK_ENTRY(widget))) > 0);
 }
 
 static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
@@ -1539,7 +1513,7 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
     priv->autonaming_enabled = TRUE;
     priv->should_show_folder_button = TRUE;
     priv->should_show_location = TRUE;
-    priv->stub_name = priv->ext_name = NULL;
+    priv->stub_name = priv->ext_name = priv->infobanner_message = NULL;
     priv->action = GTK_FILE_CHOOSER_ACTION_OPEN;
     priv->action_button =
         gtk_dialog_add_button(GTK_DIALOG(self),
@@ -1556,7 +1530,7 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
     priv->entry_name = gtk_entry_new();
     
     priv->caption_control_name =
-        hildon_caption_new(size_group, _("ckdg_ti_select_file"),
+        hildon_caption_new(size_group, "caption control name",
                            priv->entry_name, NULL,
                            HILDON_CAPTION_OPTIONAL);
     hildon_caption_set_separator(HILDON_CAPTION(priv->caption_control_name),
@@ -1566,9 +1540,8 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
           g_signal_connect( priv->entry_name, "changed",
 		          G_CALLBACK( hildon_file_chooser_entry_changed ),
 		          self );
-    g_signal_connect(priv->entry_name, "insert-text",
-		          G_CALLBACK( hildon_file_chooser_dialog_check_limit ),
-              self );
+    g_signal_connect(priv->entry_name, "invalid-input",
+		     G_CALLBACK(chooser_entry_invalid_input_cb), self);
 
     priv->hbox_location = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
     priv->hbox_items = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
@@ -1831,18 +1804,17 @@ gchar *hildon_file_chooser_dialog_get_safe_folder(
 {
   GtkFileSystem *fs;
   GtkFilePath *path;
-  gchar *result;
+  gchar *result = NULL;
 
   g_return_val_if_fail (HILDON_IS_FILE_CHOOSER_DIALOG(self), NULL);
 
   fs = _hildon_file_system_model_get_file_system(self->priv->model);
   g_object_get(self->priv->model, "safe-folder", &path, NULL);
-  if (path == NULL)
-    path = _hildon_file_system_path_for_location(fs,
-             HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE, FALSE);
 
-  result = gtk_file_system_path_to_filename(fs, path);
-  gtk_file_path_free(path);
+  if (path) {
+    result = gtk_file_system_path_to_filename(fs, path);
+    gtk_file_path_free(path);
+  }
   
   return result;
 }
@@ -1860,18 +1832,17 @@ gchar *hildon_file_chooser_dialog_get_safe_folder_uri(
 {
   GtkFileSystem *fs;
   GtkFilePath *path;
-  gchar *result;
+  gchar *result = NULL;
 
   g_return_val_if_fail (HILDON_IS_FILE_CHOOSER_DIALOG(self), NULL);
 
   fs = _hildon_file_system_model_get_file_system(self->priv->model);
   g_object_get(self->priv->model, "safe-folder", &path, NULL);
-  if (path == NULL)
-    path = _hildon_file_system_path_for_location(fs,
-             HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE, FALSE);
 
-  result = gtk_file_system_path_to_uri(fs, path);
-  gtk_file_path_free(path);
+  if (path) {
+    result = gtk_file_system_path_to_uri(fs, path);
+    gtk_file_path_free(path);
+  }
   
   return result;
 }
@@ -1888,3 +1859,47 @@ void hildon_file_chooser_dialog_focus_to_input(HildonFileChooserDialog *d)
   hildon_file_chooser_dialog_select_text(d->priv);
 }
 
+
+
+/**
+ * hildon_file_chooser_dialog_set_show_upnp:
+ * @self: a #HildonFileChooserDialog widget.
+ * @value: a gboolean value to be set.
+ * 
+ * Set whether the dialog shows UPNP locations.
+ *
+ */
+void hildon_file_chooser_dialog_set_show_upnp(
+  HildonFileChooserDialog *self, gboolean value)
+{
+  HildonFileChooserDialogPrivate *priv;
+  g_return_if_fail(HILDON_IS_FILE_CHOOSER_DIALOG(self));
+
+  priv = self->priv;
+  g_assert(HILDON_IS_FILE_SELECTION(priv->filetree));
+								     
+  g_object_set (priv->filetree, "show-upnp", value, NULL);
+}
+
+
+/**
+ * hildon_file_chooser_dialog_get_show_upnp:
+ * @self: a #HildonFileChooserDialog widget.
+ * 
+ * Gets whether the dialog shows UPNP locations.
+ * Returns: gboolean value..
+ *
+ */
+gboolean hildon_file_chooser_dialog_get_show_upnp (HildonFileChooserDialog *self)
+{
+  HildonFileChooserDialogPrivate *priv;
+  gboolean show_upnp;
+  
+  g_return_val_if_fail(HILDON_IS_FILE_CHOOSER_DIALOG(self), FALSE);
+
+  priv = self->priv;
+  g_assert(HILDON_IS_FILE_SELECTION(priv->filetree));
+								     
+  g_object_get (priv->filetree, "show-upnp", &show_upnp, NULL);
+  return show_upnp;
+}
