@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm
  *
- * Copyright (C) 2005-2006 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -54,7 +53,9 @@ enum
   PROP_SHOW_TABS = 1,
   PROP_ADDITIONAL_TAB,
   PROP_ADDITIONAL_TAB_LABEL,
-  PROP_MODEL
+  PROP_MODEL,
+  PROP_ENABLE_READONLY_CHECKBOX,
+  PROP_SHOW_TYPE_ICON
 };
 
 struct _HildonFileDetailsDialogPrivate {
@@ -65,6 +66,7 @@ struct _HildonFileDetailsDialogPrivate {
     GtkWidget *file_date, *file_time;
     GtkWidget *file_readonly, *file_device;
     GtkWidget *file_location_image, *file_device_image;
+    GtkWidget *file_image;
     GtkWidget *ok_button;
 
     GtkTreeRowReference *active_file;
@@ -190,10 +192,7 @@ static void change_state(HildonFileDetailsDialog *self, gboolean readonly)
           GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
     }
 
-    if (result == GNOME_VFS_ERROR_NO_SPACE)
-      gtk_infoprint(GTK_WINDOW(self), dgettext("ke-recv",
-                                               "cerm_device_memory_full"));
-    else if (result != GNOME_VFS_OK)
+    if (result != GNOME_VFS_OK)
       gtk_infoprint(GTK_WINDOW(self), gnome_vfs_result_to_string(result));
     
     gnome_vfs_file_info_unref(info);
@@ -273,7 +272,8 @@ hildon_file_details_dialog_class_init(HildonFileDetailsDialogClass * klass)
                                    g_param_spec_object("additional-tab",
                                    "Additional tab",
                                    "Tab to show additinal information",
-						       GTK_TYPE_WIDGET, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+			           GTK_TYPE_WIDGET,
+                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   /**
    * HildonFileDetailsDialog:show_tabs:
    *
@@ -295,10 +295,34 @@ hildon_file_details_dialog_class_init(HildonFileDetailsDialogClass * klass)
                                    "Label to the additional tab",
                                    NULL, G_PARAM_READWRITE));
 
-  g_object_class_install_property( gobject_class, PROP_MODEL,
+  g_object_class_install_property(gobject_class, PROP_MODEL,
                  g_param_spec_object("model", "Model", 
                  "HildonFileSystemModel to use when fetching information",
                  HILDON_TYPE_FILE_SYSTEM_MODEL, G_PARAM_READWRITE));
+
+  /**
+   * HildonFileDetailsDialog:enable_read_only_checkbox:
+   *
+   * Whether or not to enable the read-only checkbox. 
+   */
+  g_object_class_install_property(gobject_class,
+                 PROP_ENABLE_READONLY_CHECKBOX,
+                 g_param_spec_boolean("enable-read-only-checkbox",
+                 "Enable read-only checkbox", 
+                 "Whether or not to enable the read-only checkbox.",
+                 TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+  /**
+   * HildonFileDetailsDialog:show-type-icon
+   *
+   * Whether or not to show the file icon next to the file type
+   */
+  g_object_class_install_property(gobject_class,
+                 PROP_SHOW_TYPE_ICON,
+                 g_param_spec_boolean("show-type-icon",
+                 "Show file type icon", 
+                 "Whether or not to show the file icon next to the file type.",
+                 FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 static void
@@ -307,7 +331,7 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
     GtkWidget *caption_location, *caption_name, *caption_type;
     GtkWidget *caption_date, *caption_size, *caption_time;
     GtkWidget *caption_read, *caption_device;
-    GtkWidget *hbox_location, *hbox_device;
+    GtkWidget *hbox_location, *hbox_device, *hbox_type;
     GtkWidget *vbox;
     GtkWidget *scroll;
     GtkSizeGroup *group;
@@ -316,9 +340,9 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
     HildonFileDetailsDialogPrivate *priv;
 
     /* Initialize the private property */
-    self->priv = priv =
-      G_TYPE_INSTANCE_GET_PRIVATE(self, \
-          HILDON_TYPE_FILE_DETAILS_DIALOG, HildonFileDetailsDialogPrivate);
+    self->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE(self,
+                                HILDON_TYPE_FILE_DETAILS_DIALOG,
+                                HildonFileDetailsDialogPrivate);
 
     priv->notebook = GTK_NOTEBOOK(gtk_notebook_new());
     scroll = gtk_scrolled_window_new(NULL, NULL);
@@ -332,32 +356,51 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
 
     priv->file_device = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
     priv->file_location = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
+    gtk_label_set_ellipsize(GTK_LABEL(priv->file_location), PANGO_ELLIPSIZE_END);
     priv->file_name = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
+    gtk_label_set_ellipsize(GTK_LABEL(priv->file_name), PANGO_ELLIPSIZE_END);
     priv->file_type = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
     priv->file_size = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
     priv->file_date = g_object_new(GTK_TYPE_LABEL,"xalign", 0.0f, NULL);
     priv->file_time = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
     priv->file_readonly = gtk_check_button_new();
 
+    hbox_type = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
     hbox_location = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
     hbox_device = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
-
+    
+    priv->file_image = gtk_image_new ();
     priv->file_location_image = gtk_image_new();
     priv->file_device_image = gtk_image_new();
 
-    gtk_box_pack_start(GTK_BOX(hbox_location), priv->file_location_image, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox_type), priv->file_image,
+                       FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox_type), priv->file_type, TRUE, TRUE, 0);
+
+    gtk_box_pack_start(GTK_BOX(hbox_location), priv->file_location_image,
+                       FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox_location), priv->file_location, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox_device), priv->file_device_image, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox_device), priv->file_device_image, FALSE,
+                       TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox_device), priv->file_device, TRUE, TRUE, 0);
 
     /* Create captions for the dialog */
     caption_name = hildon_caption_new(group, _("ckdg_fi_properties_name_prompt"), 
       priv->file_name, NULL, HILDON_CAPTION_OPTIONAL);
-    caption_type = hildon_caption_new(group, _("ckdg_fi_properties_type_prompt"), 
-      priv->file_type, NULL, HILDON_CAPTION_OPTIONAL);
-    caption_location = hildon_caption_new(group, _("sfil_fi_properties_location_prompt"), 
-      hbox_location, NULL, HILDON_CAPTION_OPTIONAL);
-    caption_device = hildon_caption_new(group, _("sfil_fi_properties_device_prompt"), 
+#if 0
+    caption_type = hildon_caption_new(group,
+				      _("ckdg_fi_properties_type_prompt"), 
+				      priv->file_type, NULL, HILDON_CAPTION_OPTIONAL);
+#else
+    caption_type = hildon_caption_new(group,
+				      _("ckdg_fi_properties_type_prompt"), 
+				      hbox_type, NULL, HILDON_CAPTION_OPTIONAL);
+#endif
+    caption_location = hildon_caption_new(group,
+                               _("sfil_fi_properties_location_prompt"), 
+                               hbox_location, NULL, HILDON_CAPTION_OPTIONAL);
+    caption_device = hildon_caption_new(group,
+                               _("sfil_fi_properties_device_prompt"), 
       hbox_device, NULL, HILDON_CAPTION_OPTIONAL);
     caption_date = hildon_caption_new(group, _("ckdg_fi_properties_date_prompt"), 
       priv->file_date, NULL, HILDON_CAPTION_OPTIONAL);
@@ -438,9 +481,9 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
 }
 
 static void
-hildon_file_details_dialog_set_property( GObject *object, guint param_id,
-			                                   const GValue *value,
-                                         GParamSpec *pspec )
+hildon_file_details_dialog_set_property(GObject *object, guint param_id,
+                                        const GValue *value,
+                                        GParamSpec *pspec)
 {
   HildonFileDetailsDialogPrivate *priv;
   GtkNotebook *notebook;
@@ -507,6 +550,20 @@ hildon_file_details_dialog_set_property( GObject *object, guint param_id,
       }  
       break;
     }
+    case PROP_ENABLE_READONLY_CHECKBOX:
+    {
+      gtk_widget_set_sensitive(priv->file_readonly,
+                               g_value_get_boolean(value));
+      break;
+    }
+    case PROP_SHOW_TYPE_ICON:
+    {
+      if (g_value_get_boolean(value))
+	gtk_widget_show (priv->file_image);
+      else
+	gtk_widget_hide (priv->file_image);
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
       break;
@@ -536,6 +593,9 @@ hildon_file_details_dialog_get_property( GObject *object, guint param_id,
     case PROP_MODEL:
       g_value_set_object(value, priv->model);
       break;
+    case PROP_SHOW_TYPE_ICON:
+      g_value_set_boolean (value, GTK_WIDGET_VISIBLE (priv->file_image));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
       break;
@@ -675,23 +735,23 @@ void hildon_file_details_dialog_set_file_iter(HildonFileDetailsDialog *self, Gtk
   g_object_set(self->priv->file_type, "label", _(mime), NULL);
 
   if (size < 1024)
-    g_snprintf(buffer, sizeof(buffer), _("sfil_li_size_kb"), 1);
+    g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_kb"), 1);
   else if (size < 100 * 1024)
-    g_snprintf(buffer, sizeof(buffer), _("sfil_li_size_1kb_99kb"),
+    g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_1kb_99kb"),
                size / 1024);
   else if (size < 1024 * 1024)
-    g_snprintf(buffer, sizeof(buffer), _("sfil_li_size_100kb_1mb"),
-               size / (1024.0f * 1024.0f));
+    g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_100kb_1mb"),
+               size / 1024);
   else if (size < 10 * 1024 * 1024)
-    g_snprintf(buffer, sizeof(buffer), _("sfil_li_size_1mb_10mb"),
+    g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_1mb_10mb"),
                size / (1024.0f * 1024.0f));
   else if (size < 1024 * 1024 * 1024)
-    g_snprintf(buffer, sizeof(buffer), _("sfil_li_size_10mb_1gb"),
-               size / (1024 * 1024));
+    g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_10mb_1gb"),
+               size / (1024.0f * 1024.0f));
   else
     /* Following calculation doesn't fit into 32 bits if the
      * filesize is larger than 2^62 ;) */
-    g_snprintf(buffer, sizeof(buffer), _("sfil_li_size_1gb_or_greater"),
+    g_snprintf(buffer, sizeof(buffer), _("ckdg_va_properties_size_1gb_or_greater"),
                size / (1024.0f * 1024.0f * 1024.0f));
 
   g_object_set(self->priv->file_size, "label", buffer, NULL);
@@ -713,6 +773,21 @@ void hildon_file_details_dialog_set_file_iter(HildonFileDetailsDialog *self, Gtk
   strftime(buffer, sizeof(buffer), fmt, time_struct);
   g_object_set(self->priv->file_date, "label", buffer, NULL);
 
+  {
+    GdkPixbuf *icon;
+
+    gtk_tree_model_get
+      (model, iter,
+       HILDON_FILE_SYSTEM_MODEL_COLUMN_ICON, &icon,
+       -1);
+    if (icon)
+      {
+	gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->file_image),
+				   icon);
+	g_object_unref (icon);
+      }
+  }
+      
   /* Parent information */
   if (gtk_tree_model_iter_parent(model, &parent_iter, iter))
   {
