@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005-2006 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,43 +37,14 @@
 #include "hildon-file-system-settings.h"
 
 #include "hildon-file-common-private.h"
+#include "hildon-file-system-local-device.h"
+#include "hildon-file-system-mmc.h"
+#include "hildon-file-system-upnp.h"
+#include "hildon-file-system-obex.h"
+#include "hildon-file-system-old-gateway.h"
 
 extern GtkFileSystem *gtk_file_system_unix_new();
 
-typedef struct
-{
-  const char *icon_name;
-  const char *display_name;
-  char *path;
-} LocationType; 
-
-/* Keep this table in sync with public enumeration found in
-    hildon-file-system-common.h */
-static LocationType locations[] = {
-  { NULL, NULL, NULL }, 
-  { "qgn_list_gene_unknown_file", NULL, NULL }, 
-  { "qgn_list_filesys_common_fldr", NULL, NULL }, 
-  { "qgn_list_filesys_image_fldr", N_("sfil_li_folder_images"), NULL },
-  { "qgn_list_filesys_video_fldr", N_("sfil_li_folder_video_clips"), NULL },
-  { "qgn_list_filesys_audio_fldr", N_("sfil_li_folder_sound_clips"), NULL },
-  { "qgn_list_filesys_doc_fldr", N_("sfil_li_folder_documents"), NULL },
-  { "qgn_list_filesys_games_fldr", N_("sfil_li_folder_games"), NULL },
-  { "qgn_list_filesys_mmc_root", N_("sfil_li_mmc_localdevice"), NULL },
- /* This is used, if GConf for some reason don't contain name */
-  { "qgn_list_filesys_divc_gw", N_("sfil_li_gateway_root"), NULL },
-  { "qgn_list_filesys_divc_cls", N_("sfil_li_folder_root"), NULL }
-};
-
-/* Paths for system folders and devices */
-static const char *LOCAL_DEVICE_ROOT = "MyDocs";    /* Under $HOME */
-static const char *IMAGES_SAFE = ".images";         /* Under local device */
-static const char *VIDEOS_SAFE = ".videos";
-static const char *SOUNDS_SAFE = ".sounds";
-static const char *DOCUMENTS_SAFE = ".documents";
-static const char *GAMES_SAFE = ".games";
-
-static const char *mmc2_path = NULL;
-
 /* Let's make sure that we survise with folder names both ending and
  * not ending to slash */
 gboolean _hildon_file_system_compare_ignore_last_separator(const char *a,
@@ -96,85 +66,212 @@ gboolean _hildon_file_system_compare_ignore_last_separator(const char *a,
     return g_ascii_strncasecmp(a, b, len_a) == 0;
 }
 
-void _hildon_file_system_ensure_locations(void)
+/*** Setting up devices ************************************************************/
+
+/* If safe folders would be dynamic locations under local device, this logic
+   could be moved into local device class */
+static gchar *get_local_device_root_path(void)
 {
-  const gchar *env;
-  gchar *root;
+    const gchar *env;
 
-  if (locations[HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE].path != NULL)
-    return;
-
-  env = g_getenv("MYDOCSDIR");
-  if (env && env[0])
-    root = g_strdup(env);
-  else
-  {
-    /* g_getenv uses $HOME as last resort. Normally it returns home 
-        directory defined in passwd database. We want to use $HOME
-        when possible. */
-    env = g_getenv("HOME");
+    env = g_getenv("MYDOCSDIR");
     if (env && env[0])
-      root = g_build_path(G_DIR_SEPARATOR_S, env, LOCAL_DEVICE_ROOT, NULL);
-    else
-      root = g_build_path(G_DIR_SEPARATOR_S, g_get_home_dir(),
-                          LOCAL_DEVICE_ROOT, NULL);
-  }
+        return g_strdup(env);
 
-  locations[HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE].path = root;
-  locations[HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_IMAGES].path = 
-    g_build_path(G_DIR_SEPARATOR_S, root, IMAGES_SAFE, NULL);
-  locations[HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_VIDEOS].path = 
-    g_build_path(G_DIR_SEPARATOR_S, root, VIDEOS_SAFE, NULL);
-  locations[HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_SOUNDS].path = 
-    g_build_path(G_DIR_SEPARATOR_S, root, SOUNDS_SAFE, NULL);
-  locations[HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_DOCUMENTS].path = 
-    g_build_path(G_DIR_SEPARATOR_S, root, DOCUMENTS_SAFE, NULL);
-  locations[HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_GAMES].path =
-    g_build_path(G_DIR_SEPARATOR_S, root, DOCUMENTS_SAFE, GAMES_SAFE, NULL);
+    /* g_getenv uses $HOME as last resort. Normally it returns home 
+       directory defined in passwd database. We want to use $HOME
+       when possible. */
+    env = g_getenv("HOME");
 
-  env = g_getenv("MMC_MOUNTPOINT");
-  if (env == NULL || env[0] == 0)
-    env = "/media/mmc1";    /* Fallback location */
-
-  locations[HILDON_FILE_SYSTEM_MODEL_MMC].path = g_strdup(env);
-
-  env = g_getenv("INTERNAL_MMC_MOUNTPOINT");
-  if (env != NULL)
-  {
-    locations[HILDON_FILE_SYSTEM_MODEL_MMC].display_name =
-      N_("sfil_li_memorycard_internal");
-    mmc2_path = g_strdup(env);
-  }
+    return g_build_path(G_DIR_SEPARATOR_S, 
+              (env && env[0]) ? env : g_get_home_dir(), 
+              "MyDocs", NULL);
 }
 
-gint _hildon_file_system_get_special_location(GtkFileSystem *fs,
-                                              const GtkFilePath *path)
+static GNode *setup_safe_folder(GNode *parent, const gchar *root_path, 
+    const gchar *relative_path, const gchar *title, const gchar *icon,
+    HildonFileSystemModelItemType type)
 {
-  gchar *local_path;
-  gint i, result = HILDON_FILE_SYSTEM_MODEL_UNKNOWN;
+    HildonFileSystemSpecialLocation *location;
+    gchar *local_path, *uri;
+    GNode *result = NULL;
 
-  local_path = gtk_file_system_path_to_filename(fs, path);
-  if (local_path)
-  {
-    if (mmc2_path != NULL && strcmp(local_path, mmc2_path) == 0)
-    {
-      g_free(local_path);  
-      return -1;
+    g_assert(parent != NULL);
+
+    local_path = g_build_path(G_DIR_SEPARATOR_S, root_path, relative_path, NULL);
+    uri = g_filename_to_uri(local_path, NULL, NULL);
+
+    if (uri) {
+        location = g_object_new(HILDON_TYPE_FILE_SYSTEM_SPECIAL_LOCATION, NULL);
+        hildon_file_system_special_location_set_icon(location, icon);
+        hildon_file_system_special_location_set_display_name(location, title);
+        location->basepath = uri;
+        location->sort_weight = SORT_WEIGHT_FOLDER;
+        location->compatibility_type = type;
+        result = g_node_new(location);
+        g_node_append(parent, result);
     }
 
-    for (i = 3; i < G_N_ELEMENTS(locations); i++)
-      if (locations[i].path && 
-          _hildon_file_system_compare_ignore_last_separator(local_path,
-                                                      locations[i].path))
-      {
-        result = i;
-        break;
-      }
+    g_free(local_path);
 
-    g_free(local_path);  
-  }
+    return result;
+}
 
-  return result;
+static void setup_mmc(GNode *parent, const gchar *path, gboolean internal)
+{
+    HildonFileSystemSpecialLocation *location;
+    gchar *uri;
+
+    if (path && path[0]) {
+        GValue val = {0, };
+        g_value_init(&val, G_TYPE_BOOLEAN);
+
+        uri = g_filename_to_uri(path, NULL, NULL);
+        if (uri) {
+            location = g_object_new(HILDON_TYPE_FILE_SYSTEM_MMC, NULL);
+            g_value_set_boolean(&val, internal);
+            g_object_set_property(G_OBJECT(location), "internal-card",
+                                  &val);
+            location->basepath = uri;
+            g_node_append_data(parent, location);
+        }
+    }
+}
+
+GNode *_hildon_file_system_get_locations(GtkFileSystem *fs)
+{
+    static GNode *locations = NULL;
+
+    if (G_UNLIKELY(locations == NULL))
+    {
+        HildonFileSystemSpecialLocation *location;
+        const gchar *env;
+        gchar *rootpath;
+        GNode *rootnode;
+
+        locations = g_node_new(NULL); /* Invisible root node above everything else */
+        rootpath =  get_local_device_root_path();
+
+        /* Setup local device */
+        location = g_object_new(HILDON_TYPE_FILE_SYSTEM_LOCAL_DEVICE, NULL);
+        location->basepath = g_filename_to_uri(rootpath, NULL, NULL);
+        rootnode = g_node_new(location);
+        g_node_append(locations, rootnode);
+
+        /* Setup safe folders */
+        setup_safe_folder(rootnode, rootpath, ".images", 
+            _("sfil_li_folder_images"), "qgn_list_filesys_image_fldr",
+            HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_IMAGES);
+        setup_safe_folder(rootnode, rootpath, ".videos", 
+            _("sfil_li_folder_video_clips"), "qgn_list_filesys_video_fldr",
+            HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_VIDEOS);
+        setup_safe_folder(rootnode, rootpath, ".sounds", 
+            _("sfil_li_folder_sound_clips"), "qgn_list_filesys_audio_fldr",
+            HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_SOUNDS);
+        setup_safe_folder(rootnode, rootpath, ".documents", 
+            _("sfil_li_folder_documents"), "qgn_list_filesys_doc_fldr",
+            HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_DOCUMENTS);
+        setup_safe_folder(rootnode, rootpath, ".games", 
+            _("sfil_li_folder_games"), "qgn_list_filesys_games_fldr",
+            HILDON_FILE_SYSTEM_MODEL_SAFE_FOLDER_GAMES);
+
+        g_free(rootpath);
+
+        /* Setup MMC device(s) */
+        setup_mmc(rootnode, g_getenv("MMC_MOUNTPOINT"), FALSE);
+        setup_mmc(rootnode, g_getenv("INTERNAL_MMC_MOUNTPOINT"), TRUE);
+
+        /* Setup uPnP devices */
+        env = g_getenv("UPNP_ROOT");
+        if (env && env[0]) {
+            location = g_object_new(HILDON_TYPE_FILE_SYSTEM_UPNP, NULL);
+            location->basepath = g_strdup(env);
+            g_node_append_data(locations, location);
+        }    
+
+        if (!g_getenv("DISABLE_GATEWAY")) {
+            /* Setup gateway */
+            location = g_object_new(HILDON_TYPE_FILE_SYSTEM_OLD_GATEWAY, NULL);
+            g_node_append_data(locations, location);
+        } else {
+            /* Setup multiple Bluetooth device support */
+            env = g_getenv("HILDON_FM_OBEX_ROOT");
+            if (env && env[0]) {
+                location = g_object_new(HILDON_TYPE_FILE_SYSTEM_OBEX, NULL);
+                location->basepath = g_strdup(env);
+                g_node_append_data(locations, location);
+            }    
+        }
+    }
+
+    return locations;
+}
+
+typedef struct {
+    gchar *uri;
+    gint len_uri;
+    HildonFileSystemSpecialLocation *result;
+    gboolean is_child;
+} CallbackData;
+
+static gboolean get_special_location_callback(GNode *node, gpointer data)
+{
+    HildonFileSystemSpecialLocation *candidate = node->data;
+    CallbackData *searched = data;
+
+    if (candidate) { 
+        /* Check if the searched uri exactly matches this location OR
+          is under this location. It might be a dynamic device in that case */
+        gint len_cand = strlen(candidate->basepath);
+
+        if (len_cand > 1 && candidate->basepath[len_cand - 1] == G_DIR_SEPARATOR)
+          len_cand--;
+
+        if (searched->len_uri >= len_cand && g_ascii_strncasecmp(searched->uri, 
+                candidate->basepath, len_cand) == 0) 
+        {
+            if (searched->len_uri == len_cand) {
+                searched->result = g_object_ref(candidate);
+                searched->is_child = FALSE;
+            } else if (searched->uri[len_cand] == G_DIR_SEPARATOR) {
+                searched->result =
+                    hildon_file_system_special_location_create_child_location(
+                    candidate, searched->uri);
+                searched->is_child = TRUE;
+                ULOG_INFO("Checking if %s considers %s as dynamic device? Result = %p",
+                    candidate->basepath, searched->uri, (gpointer) searched->result);
+            }
+
+            return searched->result != NULL;
+        }
+    }
+
+    return FALSE;
+}
+
+HildonFileSystemSpecialLocation *
+_hildon_file_system_get_special_location(GtkFileSystem *fs,
+                                         const GtkFilePath *path)
+{
+    CallbackData data;
+    GNode *locations;
+
+    locations = _hildon_file_system_get_locations(fs);
+    data.uri = gtk_file_system_path_to_uri(fs, path);
+    data.result = NULL;
+    
+    if (data.uri) {
+        /* Let's precalculate the length for the entire search */
+        data.len_uri = strlen(data.uri);
+        if (data.len_uri > 1 && data.uri[data.len_uri - 1] == G_DIR_SEPARATOR)
+          data.len_uri--;
+
+        g_node_traverse(locations, G_PRE_ORDER, G_TRAVERSE_ALL, -1, 
+            get_special_location_callback, &data);
+        g_free(data.uri);
+    }
+
+    return data.result;
 }
 
 typedef struct
@@ -314,48 +411,19 @@ GdkPixbuf *_hildon_file_system_load_icon_cached(GtkIconTheme *theme,
 GdkPixbuf *
 _hildon_file_system_create_image(GtkFileSystem *fs, 
       GtkWidget *ref_widget, GtkFilePath *path, 
-      HildonFileSystemModelItemType type, gint size, gboolean mmc2)
+      HildonFileSystemSpecialLocation *location, gint size)
 {
-    GdkPixbuf *pixbuf = NULL;
+    if (!ref_widget)
+        return NULL;
 
-    if (ref_widget)
-    {
-      if (type <= HILDON_FILE_SYSTEM_MODEL_FOLDER)    
-      {
-        pixbuf = gtk_file_system_render_icon(fs, path, ref_widget, size, NULL);
-      }
-      else if (type == HILDON_FILE_SYSTEM_MODEL_GATEWAY)
-      {
-        GtkFileSystemVolume *vol;
-
-        /* We do need the path. Every gateway device has different one */
-        g_assert(path != NULL);
-
-        vol = _hildon_file_system_get_volume_for_location(fs, 
-            HILDON_FILE_SYSTEM_MODEL_GATEWAY, path, mmc2);
-
-        if (vol)
-        {
-          pixbuf = gtk_file_system_volume_render_icon(fs, vol, ref_widget, size, NULL);
-          gtk_file_system_volume_free(fs, vol);
-        }
-      }  
-
-      if (!pixbuf)
-      {
-        if (!mmc2)
-          pixbuf = _hildon_file_system_load_icon_cached(
-                     gtk_icon_theme_get_default(), 
-                     locations[type].icon_name, size);
-        else
-          pixbuf = _hildon_file_system_load_icon_cached(
-                     gtk_icon_theme_get_default(),     
-                     locations[HILDON_FILE_SYSTEM_MODEL_MMC].icon_name, size);
-
-      }
+    if (location) {
+        GdkPixbuf *pixbuf;
+        pixbuf = hildon_file_system_special_location_get_icon(location,
+               fs, ref_widget, size);
+        if (pixbuf) return pixbuf;
     }
 
-    return pixbuf;
+    return gtk_file_system_render_icon(fs, path, ref_widget, size, NULL);
 }
 
 static const gchar *get_custom_root_name(const GtkFilePath *path)
@@ -381,126 +449,52 @@ static const gchar *get_custom_root_name(const GtkFilePath *path)
 
 gchar *
 _hildon_file_system_create_file_name(GtkFileSystem *fs,
-  const GtkFilePath *path, HildonFileSystemModelItemType type,
-  GtkFileInfo *info, gboolean mmc2)
+  const GtkFilePath *path, HildonFileSystemSpecialLocation *location, 
+  GtkFileInfo *info)
 {
-  gchar *str;
-
-  if (type == HILDON_FILE_SYSTEM_MODEL_GATEWAY ||
-      type == HILDON_FILE_SYSTEM_MODEL_MMC || mmc2)
-  {
-    GtkFileSystemVolume *vol;
-
-    vol = _hildon_file_system_get_volume_for_location(fs, type, path, mmc2);
-
-    if (vol)
-    {
-      str = gtk_file_system_volume_get_display_name(fs, vol);
-      gtk_file_system_volume_free(fs, vol);
-
-      /* We return device names, unless it's unnamed mmc */
-      if (str) 
-      {
-        g_strstrip(str);
-
-        if (str[0] && strcmp(str, "mmc-undefined-name") == 0) 
-        {
-          g_free(str);
-          return g_strdup(_("sfil_li_mmc_localdevice"));
-        }
-        else if (str[0])
-        {
-          return str;
-        }
-
-        g_free(str);
-      }
-    }
-  }  
-  else if (type == HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE)
-  {
-    HildonFileSystemSettings *settings;
-    gchar *name;
-    
-    settings = _hildon_file_system_settings_get_instance();
-    g_object_get(settings, "btname", &name, NULL);
-
-    if (name)
-    {
-      if (name[0]) return name;
-      g_free(name);
-    }
-  }
-
-  if (mmc2)
-  {
-    const gchar *name;
-    name = _("sfil_li_memorycard_removable");
-    g_return_val_if_fail(name, NULL);
-
-    return g_strdup(name);
-  }
-
-  if (type > HILDON_FILE_SYSTEM_MODEL_FOLDER)
-  {
-    const gchar *name;
-    g_return_val_if_fail(locations[type].display_name, NULL);
-    name = _(locations[type].display_name);
-    g_return_val_if_fail(name, NULL);
-
-    return g_strdup(name);
+  if (location) {
+    char *name;
+    name = hildon_file_system_special_location_get_display_name(location, fs);
+    if (name) return name;
   }
 
   if (info)  
-  {
-    const gchar *name;
-    g_return_val_if_fail(info, NULL);
-    name = gtk_file_info_get_display_name(info);
-    g_return_val_if_fail(name, NULL);
-
-    return g_strdup(name);
-  }
+    return g_strdup(gtk_file_info_get_display_name(info));
 
   return g_strdup(get_custom_root_name(path));
 }
 
 gchar *
-_hildon_file_system_create_display_name(GtkFileSystem *fs, 
-  const GtkFilePath *path, HildonFileSystemModelItemType type, 
-  GtkFileInfo *info, gboolean mmc2)
+_hildon_file_system_create_display_name(GtkFileSystem *fs,
+  const GtkFilePath *path, HildonFileSystemSpecialLocation *location, 
+  GtkFileInfo *info)
 {
   gchar *str, *dot;
-  const gchar *mime_type = NULL;
 
-  str = _hildon_file_system_create_file_name(fs, path, type, info, mmc2);   
+  str = _hildon_file_system_create_file_name(fs, path, location, info);   
 
-  if (info)
-    mime_type = gtk_file_info_get_mime_type(info);
-
-  if (type < HILDON_FILE_SYSTEM_MODEL_FOLDER)
+  if (info && !gtk_file_info_get_is_folder(info))
   {
-     dot = _hildon_file_system_search_extension(str, mime_type);
-     if (dot && dot != str)
-       *dot = 0;
+      dot = _hildon_file_system_search_extension(str, 
+                gtk_file_info_get_mime_type(info));
+      if (dot && dot != str)
+        *dot = 0;
   }
+
   return str;
 }
 
 GtkFilePath *_hildon_file_system_path_for_location(GtkFileSystem *fs, 
-  HildonFileSystemModelItemType type, gboolean mmc2)
+  HildonFileSystemSpecialLocation *location)
 {
-  g_assert(0 <= type && type < G_N_ELEMENTS(locations));
-  if (!mmc2)
-    return gtk_file_system_filename_to_path(fs, locations[type].path);
-  else
-    return gtk_file_system_filename_to_path(fs, mmc2_path);
+  g_assert(HILDON_IS_FILE_SYSTEM_SPECIAL_LOCATION(location));
+  return gtk_file_system_uri_to_path(fs, location->basepath);
 }
 
 /* You can omit either type or base */
 GtkFileSystemVolume *
 _hildon_file_system_get_volume_for_location(GtkFileSystem *fs, 
-  HildonFileSystemModelItemType type,
-  const GtkFilePath *base, gboolean mmc2)
+    HildonFileSystemSpecialLocation *location)
 {
     GSList *volumes, *iter;
     GtkFilePath *mount_path, *path;
@@ -510,12 +504,7 @@ _hildon_file_system_get_volume_for_location(GtkFileSystem *fs,
     /* We cannot just use get_volume_for_path, because it won't
         work work with URIs other than file:// */
     volumes = gtk_file_system_list_volumes(fs);
-
-    mount_path = base ? gtk_file_path_copy(base) : 
-                 _hildon_file_system_path_for_location(fs, type, mmc2);
-
-    if (mount_path == NULL) return NULL;
-
+    mount_path = _hildon_file_system_path_for_location(fs, location);
     path_a = gtk_file_path_get_string(mount_path);
 
     for (iter = volumes; iter; iter = g_slist_next(iter))
@@ -661,14 +650,14 @@ gchar *_hildon_file_system_search_extension(gchar *name, const gchar *mime)
 
       if (name <= candidate && 
           g_ascii_strcasecmp(candidate, type->extension) == 0)
-        return candidate;
+	return candidate;
     }
   }
 
-  /* If we didn't find a match (but type was recognized), then we just
-     return the part after last dot. If we didn't have type information,
-     then we do not touch the name. */
-  return mime ? g_strrstr(name, ".") : NULL;
+  /* If we didn't find a match, then we just return the part after
+     last dot.
+  */
+  return g_strrstr(name, ".");
 }
 
 enum {
