@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +34,7 @@
 
 #include <gtk/gtkicontheme.h>
 #include "hildon-file-system-common.h"
+#include "hildon-file-system-special-location.h"
 
 G_BEGIN_DECLS
 
@@ -43,30 +43,29 @@ G_BEGIN_DECLS
 gboolean 
 _hildon_file_system_compare_ignore_last_separator(const char *a, const char *b);
 
-void _hildon_file_system_ensure_locations(void);
+GNode *_hildon_file_system_get_locations(GtkFileSystem *fs);
 
-gint _hildon_file_system_get_special_location(GtkFileSystem *fs, const GtkFilePath *path);
+HildonFileSystemSpecialLocation *
+_hildon_file_system_get_special_location(GtkFileSystem *fs, const GtkFilePath *path);
 
 GdkPixbuf *_hildon_file_system_create_image(GtkFileSystem *fs, 
       GtkWidget *ref_widget, GtkFilePath *path, 
-      HildonFileSystemModelItemType type, gint size, gboolean mmc2);
+      HildonFileSystemSpecialLocation *location, gint size);
 
 gchar *_hildon_file_system_create_file_name(GtkFileSystem *fs, 
-  const GtkFilePath *path, HildonFileSystemModelItemType type,  
-  GtkFileInfo *info, gboolean mmc2);
+  const GtkFilePath *path, HildonFileSystemSpecialLocation *location,  
+  GtkFileInfo *info);
 
 gchar *_hildon_file_system_create_display_name(GtkFileSystem *fs, 
-  const GtkFilePath *path, HildonFileSystemModelItemType type,  
-  GtkFileInfo *info, gboolean mmc2);
+  const GtkFilePath *path, HildonFileSystemSpecialLocation *location,  
+  GtkFileInfo *info);
 
 GtkFilePath *_hildon_file_system_path_for_location(GtkFileSystem *fs, 
-  HildonFileSystemModelItemType type, gboolean mmc2);
+  HildonFileSystemSpecialLocation *location);
 
-/* You can omit either type or base */
 GtkFileSystemVolume *
 _hildon_file_system_get_volume_for_location(GtkFileSystem *fs, 
-  HildonFileSystemModelItemType type,
-  const GtkFilePath *base, gboolean mmc2);
+    HildonFileSystemSpecialLocation *location);
 
 gchar *_hildon_file_system_search_extension(gchar *name, const gchar *mime);
 long _hildon_file_system_parse_autonumber(const char *start);
