@@ -1,14 +1,13 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005-2006 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.  All rights reserved.
  *
- * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
+ * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
+ * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,7 +34,6 @@
 
 #include <osso-log.h>
 #include <bt-gconf.h>
-#include <bttools-dbus.h>
 #include <gconf/gconf-client.h>
 #include <mce/dbus-names.h>
 #include <mce/mode-names.h>
@@ -44,10 +42,12 @@
 #include <libintl.h>
 #include <string.h>
 
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+
 #include "hildon-file-common-private.h"
 #include "hildon-file-system-settings.h"
 
@@ -56,16 +56,40 @@ enum {
   PROP_BTNAME,
   PROP_GATEWAY,
   PROP_USB,
-  PROP_GATEWAY_FTP
+  PROP_GATEWAY_FTP,
+  PROP_MMC_PRESENT,
+  PROP_MMC_COVER_OPEN,
+  PROP_MMC_USED,
+  PROP_MMC_CORRUPTED,
+  PROP_INTERNAL_MMC_CORRUPTED
 };
 
 #define PRIVATE(obj) HILDON_FILE_SYSTEM_SETTINGS(obj)->priv
-#define USB_CABLE_DIR "/system/osso/af"
-#define USB_CABLE_KEY USB_CABLE_DIR "/usb-cable-attached"
 
+#define USB_CABLE_DIR "/system/osso/af"
+#define MMC_DIR "/system/osso/af/mmc"
+
+#define USB_CABLE_KEY USB_CABLE_DIR "/usb-cable-attached"
+#define MMC_USED_KEY USB_CABLE_DIR "/mmc-used-over-usb"
+#define MMC_PRESENT_KEY USB_CABLE_DIR "/mmc-device-present"
+#define MMC_COVER_OPEN_KEY USB_CABLE_DIR "/mmc-cover-open"
+#define MMC_CORRUPTED_KEY MMC_DIR "/mmc-corrupted"
+#define MMC_INTERNAL_CORRUPTED_KEY MMC_DIR "/internal-mmc-corrupted"
 
 #define MCE_MATCH_RULE "type='signal',interface='" MCE_SIGNAL_IF \
                        "',member='" MCE_DEVICE_MODE_SIG "'"
+
+/* For getting and tracking the Bluetooth name
+ */
+#define BTNAME_SERVICE                  "org.bluez"
+#define BTNAME_REQUEST_IF               "org.bluez.Adapter"
+#define BTNAME_SIGNAL_IF                "org.bluez.Adapter"
+#define BTNAME_REQUEST_PATH             "/org/bluez/hci0"
+#define BTNAME_SIGNAL_PATH              "/org/bluez/hci0"
+
+#define BTNAME_REQ_GET                  "GetName"
+#define BTNAME_SIG_CHANGED              "NameChanged"
+
 #define BTNAME_MATCH_RULE "type='signal',interface='" BTNAME_SIGNAL_IF \
                           "',member='" BTNAME_SIG_CHANGED "'"
 
@@ -84,6 +108,11 @@ struct _HildonFileSystemSettingsPrivate
   gboolean gconf_ready;
   gboolean flightmode_ready;
   gboolean btname_ready;
+
+  gboolean mmc_is_present;
+  gboolean mmc_is_corrupted;
+  gboolean mmc_used_over_usb;
+  gboolean mmc_cover_open;
 };
 
 G_DEFINE_TYPE(HildonFileSystemSettings, \
@@ -114,6 +143,18 @@ hildon_file_system_settings_get_property(GObject *object,
     case PROP_GATEWAY_FTP:
       g_value_set_boolean(value, priv->gateway_ftp);
       break;
+    case PROP_MMC_PRESENT:
+      g_value_set_boolean(value, priv->mmc_is_present);
+      break;
+    case PROP_MMC_USED:
+      g_value_set_boolean(value, priv->mmc_used_over_usb);
+      break;
+    case PROP_MMC_COVER_OPEN:
+      g_value_set_boolean(value, priv->mmc_cover_open);
+      break;
+    case PROP_MMC_CORRUPTED:
+      g_value_set_boolean(value, priv->mmc_is_corrupted);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -179,6 +220,50 @@ set_gateway_from_gconf_value(HildonFileSystemSettings *self,
   g_object_notify(G_OBJECT(self), "gateway-ftp");
 }
 
+static void
+set_mmc_cover_open_from_gconf_value(HildonFileSystemSettings *self,
+		                    GConfValue *value)
+{
+  if (value && value->type == GCONF_VALUE_BOOL)
+  {
+    self->priv->mmc_cover_open = gconf_value_get_bool(value);
+    g_object_notify(G_OBJECT(self), "mmc-cover-open");
+  }
+}
+
+static void
+set_mmc_corrupted_from_gconf_value(HildonFileSystemSettings *self,
+			           GConfValue *value)
+{
+  if (value && value->type == GCONF_VALUE_BOOL)
+  {
+    self->priv->mmc_is_corrupted = gconf_value_get_bool(value);
+    g_object_notify(G_OBJECT(self), "mmc-is-corrupted");
+  }
+}
+
+static void
+set_mmc_present_from_gconf_value(HildonFileSystemSettings *self,
+	                         GConfValue *value)
+{
+  if (value && value->type == GCONF_VALUE_BOOL)
+  {
+    self->priv->mmc_is_present = gconf_value_get_bool(value);
+    g_object_notify(G_OBJECT(self), "mmc-is-present");
+  }
+}
+
+static void
+set_mmc_used_from_gconf_value(HildonFileSystemSettings *self,
+                              GConfValue *value)
+{
+  if (value && value->type == GCONF_VALUE_BOOL)
+  {
+    self->priv->mmc_used_over_usb = gconf_value_get_bool(value);
+    g_object_notify(G_OBJECT(self), "mmc-used");
+  }
+}
+
 static void
 set_usb_from_gconf_value(HildonFileSystemSettings *self,
                          GConfValue *value)
@@ -373,6 +458,7 @@ hildon_file_system_settings_setup_dbus(HildonFileSystemSettings *self)
   if (dbus_connection_send_with_reply(conn, request, &call, -1))
   {
     dbus_pending_call_set_notify(call, btname_received, self, NULL);
+    dbus_pending_call_block(call);
     dbus_pending_call_unref(call);
   }
 
@@ -412,14 +498,34 @@ gconf_gateway_changed(GConfClient *client, guint cnxn_id,
 }
 
 static void
-gconf_usb_changed(GConfClient *client, guint cnxn_id,
-                  GConfEntry *entry, gpointer data)
+gconf_mmc_value_changed(GConfClient *client, guint cnxn_id,
+		        GConfEntry *entry, gpointer data)
 {
   g_assert(entry != NULL);
-  	 
+
+  if (g_ascii_strcasecmp(entry->key, MMC_CORRUPTED_KEY) == 0)
+    set_mmc_corrupted_from_gconf_value(
+      HILDON_FILE_SYSTEM_SETTINGS(data), entry->value);
+}
+
+static void
+gconf_value_changed(GConfClient *client, guint cnxn_id,
+                    GConfEntry *entry, gpointer data)
+{
+  g_assert(entry != NULL);
+  
   if (g_ascii_strcasecmp(entry->key, USB_CABLE_KEY) == 0)
     set_usb_from_gconf_value(
       HILDON_FILE_SYSTEM_SETTINGS(data), entry->value);
+  else if (g_ascii_strcasecmp(entry->key, MMC_USED_KEY) == 0)
+         set_mmc_used_from_gconf_value(
+           HILDON_FILE_SYSTEM_SETTINGS(data), entry->value);
+  else if (g_ascii_strcasecmp(entry->key, MMC_PRESENT_KEY) == 0)
+	 set_mmc_present_from_gconf_value(
+	   HILDON_FILE_SYSTEM_SETTINGS(data), entry->value);
+  else if (g_ascii_strcasecmp(entry->key, MMC_COVER_OPEN_KEY) == 0)
+	 set_mmc_cover_open_from_gconf_value(
+	   HILDON_FILE_SYSTEM_SETTINGS(data), entry->value);
 }
 
 static void
@@ -481,6 +587,22 @@ hildon_file_system_settings_class_init(HildonFileSystemSettingsClass *klass)
     g_param_spec_boolean("gateway-ftp", "Gateway ftp",
                          "Whether current gateway device supports file transfer",
                          FALSE, G_PARAM_READABLE));
+  g_object_class_install_property(object_class, PROP_MMC_USED,
+    g_param_spec_boolean("mmc-used", "MMC used",
+                         "Whether or not the MMC is being used",
+                         FALSE, G_PARAM_READABLE));
+  g_object_class_install_property(object_class, PROP_MMC_PRESENT,
+    g_param_spec_boolean("mmc-is-present", "MMC present",
+	                 "Whether or not the MMC is present",
+			 FALSE, G_PARAM_READABLE));
+  g_object_class_install_property(object_class, PROP_MMC_CORRUPTED,
+    g_param_spec_boolean("mmc-is-corrupted", "MMC corrupted",
+	                 "Whether or not the MMC is corrupted",
+			 FALSE, G_PARAM_READABLE));
+  g_object_class_install_property(object_class, PROP_MMC_COVER_OPEN,
+    g_param_spec_boolean("mmc-cover-open", "MMC cover open",
+	                 "Whether or not the MMC cover is open",
+			 FALSE, G_PARAM_READABLE));
 }
 
 static gboolean delayed_init(gpointer data)
@@ -510,6 +632,15 @@ static gboolean delayed_init(gpointer data)
     error = NULL;
   }
 
+  gconf_client_add_dir(self->priv->gconf, MMC_DIR,
+		       GCONF_CLIENT_PRELOAD_NONE, &error);
+  if (error != NULL)
+  {
+    ULOG_ERR_F("gconf_client_add_dir failed: %s", error->message);
+    g_error_free(error);
+    error = NULL;
+  }
+  
   gconf_client_notify_add(self->priv->gconf, BTCOND_GCONF_PATH,
                           gconf_gateway_changed, self, NULL, &error);
   if (error != NULL)
@@ -520,7 +651,7 @@ static gboolean delayed_init(gpointer data)
   }
 
   gconf_client_notify_add(self->priv->gconf, USB_CABLE_DIR,
-                          gconf_usb_changed, self, NULL, &error);
+                          gconf_value_changed, self, NULL, &error);
   if (error != NULL)
   {
     ULOG_ERR_F("gconf_client_notify_add failed: %s", error->message);
@@ -528,6 +659,16 @@ static gboolean delayed_init(gpointer data)
     error = NULL;
   }
 
+  gconf_client_notify_add(self->priv->gconf, MMC_DIR,
+		          gconf_mmc_value_changed, self, NULL, &error);
+ 
+  if (error != NULL)
+  {
+    ULOG_ERR_F("gconf_client_notify_add failed: %s", error->message);
+    g_error_free(error);
+    error = NULL;
+  }
+  
   value = gconf_client_get_without_default(self->priv->gconf,
                                            BTCOND_GCONF_PREFERRED, &error);
   if (error != NULL)
@@ -558,6 +699,21 @@ static gboolean delayed_init(gpointer data)
     gconf_value_free(value);
   }
 
+  value = gconf_client_get_without_default(self->priv->gconf,
+                                           MMC_USED_KEY, &error);
+  if (error != NULL)
+  {
+    ULOG_ERR_F("gconf_client_get_without_default failed: %s",
+               error->message);
+    g_error_free(error);
+    error = NULL;
+  }
+  else if (value != NULL)
+  {
+    set_mmc_used_from_gconf_value(self, value);
+    gconf_value_free(value);
+  }
+
   self->priv->gconf_ready = TRUE;
 
   return FALSE; /* We need this only once */
@@ -595,34 +751,70 @@ gboolean _hildon_file_system_settings_ready(HildonFileSystemSettings *self)
          self->priv->flightmode_ready;
 }
 
-/* This is used to check if we can access gateway.
-   Nowadays we check ftp support as well, not just flight mode. */
-gboolean _hildon_file_system_settings_get_flight_mode(void)
+static gboolean banner_timeout(gpointer data)
 {
-  HildonFileSystemSettings *self;
-  self = _hildon_file_system_settings_get_instance();
-  return self->priv->flightmode || !self->priv->gateway_ftp;
+  guint *id = data;
+
+  ULOG_DEBUG_F("entered");
+  _hildon_file_system_cancel_banner(NULL);
+  *id = 0;
+
+  return FALSE;
 }
 
+/* The clients of delayed_infobanner service are identified by their
+   pid.  However, Linux has the bug that each thread has its own pid.
+   It might happen that the banner is cancelled from a different
+   thread that asked for it, but we need to make sure that we use the
+   same pid in both messages.
+
+   Therefore, we just store the pid of the first thread in this
+   process that uses the service, and then use it to identify
+   ourselves to the statusbar.
+
+   XXX - It is theoretically possible that the pid will be reused
+         while this process still uses it as an identifier.
+
+	 When the kernel-glibc combo is fixed to no longer have
+	 per-thread-pids, banner_pid can be removed.
+*/
+
+static dbus_int32_t banner_pid = 0;
+
 #define BANNER_SERVICE "com.nokia.statusbar"
 #define BANNER_REQUEST_PATH "/com/nokia/statusbar"
 #define BANNER_REQUEST_IF "com.nokia.statusbar"
 #define BANNER_SHOW "delayed_infobanner"
 #define BANNER_HIDE "cancel_delayed_infobanner"
+#define BANNER_TIMEOUT 500
 
 /* Communication with tasknavigator for displaying possible
    banner while making blocking calls */
-void _hildon_file_system_prepare_banner(void)
+void _hildon_file_system_prepare_banner(guint *timeout_id)
 {
   HildonFileSystemSettings *settings;
   DBusConnection *conn;
   DBusMessage *message;
-  dbus_int32_t pid;
   static const dbus_int32_t initial_value = 1000;
   static const dbus_int32_t display_timeout = 30000;
   const char *ckdg_pb_updating_str;
   dbus_bool_t ret;
 
+  if (timeout_id == NULL) return;
+
+  /* just refresh the timeout if there is already a timeout */
+  if (*timeout_id != 0)
+  {
+    ULOG_DEBUG_F("refreshing existing timeout");
+    if (!g_source_remove(*timeout_id))
+    {
+      ULOG_ERR_F("g_source_remove() failed");
+    }
+    *timeout_id = g_timeout_add(BANNER_TIMEOUT, banner_timeout,
+                                timeout_id);
+    return;
+  }
+
   ckdg_pb_updating_str = HCS("ckdg_pb_updating");
   settings = _hildon_file_system_settings_get_instance();
   g_assert(settings != NULL);
@@ -636,8 +828,10 @@ void _hildon_file_system_prepare_banner(void)
     return;
   }
 
-  pid = getpid(); /* id */
-  ret = dbus_message_append_args(message, DBUS_TYPE_INT32, &pid,
+  if (banner_pid == 0)
+    banner_pid = getpid(); /* id */
+
+  ret = dbus_message_append_args(message, DBUS_TYPE_INT32, &banner_pid,
                                  DBUS_TYPE_INT32, &initial_value,
                                  DBUS_TYPE_INT32, &display_timeout,
                                  DBUS_TYPE_STRING, &ckdg_pb_updating_str,
@@ -657,15 +851,27 @@ void _hildon_file_system_prepare_banner(void)
   {
     dbus_connection_flush(conn);
   }
+
+  ULOG_DEBUG_F("banner prepared");
+  *timeout_id = g_timeout_add(BANNER_TIMEOUT, banner_timeout, timeout_id);
+
   dbus_message_unref(message);
 }
 
-void _hildon_file_system_cancel_banner(void)
+void _hildon_file_system_cancel_banner(guint *timeout_id)
 {
   HildonFileSystemSettings *settings;
   DBusConnection *conn;
   DBusMessage *message;
-  dbus_int32_t pid;
+
+  if (timeout_id != NULL && *timeout_id != 0)
+  {
+    if (!g_source_remove(*timeout_id))
+    {
+      ULOG_ERR_F("g_source_remove() failed");
+    }
+    *timeout_id = 0;
+  }
 
   settings = _hildon_file_system_settings_get_instance();
   g_assert(settings != NULL);
@@ -679,8 +885,7 @@ void _hildon_file_system_cancel_banner(void)
     return;
   }
 
-  pid = getpid();
-  if (!dbus_message_append_args(message, DBUS_TYPE_INT32, &pid,
+  if (!dbus_message_append_args(message, DBUS_TYPE_INT32, &banner_pid,
                                 DBUS_TYPE_INVALID))
   {
     ULOG_ERR_F("dbus_message_append_args failed");
@@ -696,6 +901,7 @@ void _hildon_file_system_cancel_banner(void)
   {
     dbus_connection_flush(conn);
   }
+  ULOG_DEBUG_F("banner closed");
   dbus_message_unref(message);
 }
 
